// I2C GT911/GT915 
// Сергей Бакаев(ser8989)
// 
// Использование библиотеки Wire для чтения/записи данных из
// GT911 или GT915

// Создано март 2021 г.
// 

#include <Wire.h>
#define GOODIX_CONFIG_911_LENGTH 186

void setup() 
{
  Wire.begin();                // join i2c bus (address optional for master)
  Serial.begin(74880);         // start serial communication at 74880bps
  Serial.println("Type comand (READ or WRITE)");
}

byte GT911_CFG_TBL[GOODIX_CONFIG_911_LENGTH];

void GT911_WR_Reg(int reg, byte val)//, char *buf
{
  char addr[2] = {(char)(reg >> 8), (char)reg};
  Wire.beginTransmission(0x5d); // transmit to device #93 (0x5d)
     
  Wire.write(addr,2);// sets register pointer to echo #x register (0xXX)
  Wire.write(val);
  Wire.endTransmission();      // stop transmitting
}

boolean GT911_RD_Reg(int reg, int len)//, char *buf
{
  int chunkSize = 31;
  byte chunks = len / chunkSize + (len%chunkSize > 0 ? 1 : 0); // библиотека не может считать большее количество байт из-за внутреннего буфера
  byte tableIndex = 0;

  char addr[2] = {(char)(reg >> 8), (char)reg};  
  Wire.beginTransmission(0x5d); // transmit to device #93 (0x5d)
  
  Wire.write(addr,2);// sets register pointer to echo #x register (0xXX)
  Wire.endTransmission();// stop transmitting
 
  for (byte i=0; i<chunks; i++)
  {
    Wire.requestFrom(0x5d, chunkSize); // запрашиваем у ведомого устройства #5d 32 байта
    if (Wire.available() == 0)
      return false;

   while(1 <= Wire.available())    // ведомое устройство может отправить не все запрашиваемые байты
   { 
      if (tableIndex > len)
      {
        break;
      }
      GT911_CFG_TBL[tableIndex] = Wire.read();
      tableIndex++;        
   }     
 }
 return true;
}

void DumpConf(int reg, int len)
{
 for (byte i=0; i<len; i++)
 {
   Serial.print(i, DEC);Serial.print("\t");
   Serial.print(reg+i, HEX);Serial.print(" ");    
   Serial.print("0x");Serial.print(GT911_CFG_TBL[i],HEX);Serial.print("\t");Serial.print(GT911_CFG_TBL[i],BIN);Serial.print("b"); //Serial.print(",");
   Serial.println();        // выводим символ
 }
}

void WriteConf()
{
  Serial.println("Write configuration start");
  //GT911_WR_Reg(0x804D,0xFD);//Module_Switch1

  // set touchscreen resolution to 1024x600
  GT911_WR_Reg(0x8048,0x00);
  GT911_WR_Reg(0x8049,0x04);
  GT911_WR_Reg(0x804A,0x58);
  GT911_WR_Reg(0x804B,0x02);

  // reverse Y
  GT911_WR_Reg(0x80B7,0x02);
  GT911_WR_Reg(0x80B8,0x04);
  GT911_WR_Reg(0x80B9,0x06);
  GT911_WR_Reg(0x80BA,0x08);
  GT911_WR_Reg(0x80BB,0x0A);
  GT911_WR_Reg(0x80BC,0x0C);
  GT911_WR_Reg(0x80BD,0x0E);
  GT911_WR_Reg(0x80BE,0x10);
  GT911_WR_Reg(0x80BF,0x12);
  GT911_WR_Reg(0x80C0,0x14);

  // reverse X
  GT911_WR_Reg(0x80D5,0x00);
  GT911_WR_Reg(0x80D6,0x02);
  GT911_WR_Reg(0x80D7,0x04);
  GT911_WR_Reg(0x80D8,0x06);
  GT911_WR_Reg(0x80D9,0x08);
  GT911_WR_Reg(0x80DA,0x0A);
  GT911_WR_Reg(0x80DB,0x0C);
  GT911_WR_Reg(0x80DC,0x1D);
  GT911_WR_Reg(0x80DD,0x1E);
  GT911_WR_Reg(0x80DE,0x1F);
  GT911_WR_Reg(0x80DF,0x20);
  GT911_WR_Reg(0x80E0,0x21);
  GT911_WR_Reg(0x80E1,0x22);
  GT911_WR_Reg(0x80E2,0x24);
  GT911_WR_Reg(0x80E3,0x26);
  GT911_WR_Reg(0x80E4,0x28);
  
  byte checkSum = GetCheckSum();
  
  GT911_WR_Reg(0x80FF, checkSum);
  GT911_WR_Reg(0x8100,0x01);
  Serial.println("Write configuration end");
}

byte GetCheckSum()
{
  ReadConf(0x8047,GOODIX_CONFIG_911_LENGTH);
  byte checkSum = 0;
  for (byte i = 0; i < (GOODIX_CONFIG_911_LENGTH - 2); i++) 
    checkSum += GT911_CFG_TBL[i]; 

  checkSum = (~checkSum) + 1;

  return checkSum;  
}

boolean ReadConf(int reg, int len)
{
  Serial.println("Read configuration start");
  if (GT911_RD_Reg(reg,len))
  {
      Serial.println("Read configuration successful");
      return true;    
  }
  Serial.println("Read configuration Error!");
  return false;
}

void Backup()
{
  ReadConf(0x8047,GOODIX_CONFIG_911_LENGTH);
  for (byte i=0; i<GOODIX_CONFIG_911_LENGTH; i++)
  {
   Serial.print("0x");Serial.print(GT911_CFG_TBL[i],HEX);Serial.print(",");   
  }
  Serial.println();
}

void Restore()
{
  //byte config[] = { 0x6D,0x20,0x3,0xE0,0x1,0x5,0xD,0x0,0x1,0xA,0x28,0xF,0x50,0x32,0x3,0x5,0x0,0x0,0x0,0x0,0x0,0x0,0x8,0x17,0x19,0x1C,0x14,0x87,0x29,0xA,0x66,0x68,0xEB,0x4,0x0,0x0,0x2,0x0,0x2,0x11,0x0,0x1,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x5A,0x8C,0x94,0xC5,0x2,0x7,0x19,0x0,0x4,0x93,0x5E,0x0,0x87,0x66,0x0,0x7C,0x70,0x0,0x72,0x7A,0x0,0x6B,0x86,0x0,0x6A,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x2,0x4,0x6,0x8,0xA,0xC,0xE,0x10,0x12,0x14,0xFF,0xFF,0xFF,0xFF,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x2,0x4,0x6,0x8,0xA,0xC,0x1D,0x1E,0x1F,0x20,0x21,0x22,0x24,0x26,0x28,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x8D,0x0 };
  byte config[] = { 0xFF,0x00,0x04,0x58,0x02,0x0A,0xFD,0x00,0x01,0x08,0x28,0x08,0x5A,0x3C,0x03,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x1A,0x1E,0x14,0x87,0x29,0x0A,0x75,0x77,0xB2,0x04,0x00,0x00,0x00,0x9A,0x01,0x11,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x50,0xA0,0x94,0xD5,0x02,0x08,0x00,0x00,0x04,0xA1,0x55,0x00,0x8F,0x62,0x00,0x7F,0x71,0x00,0x73,0x82,0x00,0x69,0x95,0x00,0x69,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0x12,0x10,0x0E,0x0C,0x0A,0x08,0x06,0x04,0x02,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x28,0x26,0x24,0x22,0x21,0x20,0x1F,0x1E,0x1D,0x0C,0x0A,0x08,0x06,0x04,0x02,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5,0x00 };
  for (byte i=0; i<sizeof(config); i++)
  {
    int regAddress = 0x8047;
    GT911_WR_Reg(regAddress+i, config[i]);
  }

  Serial.print("Restore end");
}

void loop() 
{ 
  if (Serial.available()) {
    String command = Serial.readString();
    Serial.println(command);       
    if (command.equalsIgnoreCase("READ\n")){
      ReadConf(0x8047,GOODIX_CONFIG_911_LENGTH);
      DumpConf(0x8047,GOODIX_CONFIG_911_LENGTH);
    }    
    else
    if (command.equalsIgnoreCase("WRITE\n"))
    {
        WriteConf();
    }
    else
    if (command.equalsIgnoreCase("BACKUP\n"))
    {
        Backup();
    }
    else
    if (command.equalsIgnoreCase("RESTORE\n"))
    {
        Restore();
    }
  }
}  
